Type: Package
Package: CuratedAtlasQueryR
Title: Queries the Human Cell Atlas
Version: 1.10.0
Authors@R: c(
    person(
        "Stefano",
        "Mangiola",
        email = "mangiolastefano@gmail.com",
        role = c("aut", "cre", "rev"),
        comment = c(ORCID = "0000-0001-7474-836X")
    ),
    person(
        "Michael",
        "Milton",
        email = "milton.m@wehi.edu.au",
        role = c("aut", "rev"),
        comment = c(ORCID = "0000-0002-8965-2595")
    ),
    person(
        "Martin",
        "Morgan",
        email = "Martin.Morgan@RoswellPark.org",
        role = c("ctb", "rev")
    ),
    person(
        "Vincent",
        "Carey",
        email = "stvjc@channing.harvard.edu",
        role = c("ctb", "rev")
    ),
    person(
        "Julie",
        "Iskander",
        email = "iskander.j@wehi.edu.au",
        role = c( "rev")
    ),
    person(
        "Tony",
        "Papenfuss",
        email = "papenfuss@wehi.edu.au",
        role = c( "rev")
    ),
    person(
        "Silicon Valley Foundation",
        "CZF2019-002443",
        role = c( "fnd")
    ),
    person(
        "NIH NHGRI",
        "5U24HG004059-18",
        role = c( "fnd")
    ),
    person(
        "Victoria Cancer Agency",
        "ECRF21036",
        role = c( "fnd")
    ),
    person(
        "NHMRC",
        "1116955",
        role = c( "fnd")
    ))
Description: Provides access to a copy of the Human Cell Atlas, but with 
    harmonised metadata. This allows for uniform querying across numerous 
    datasets within the Atlas using common fields such as cell type, tissue 
    type, and patient ethnicity. Usage involves first querying the metadata
    table for cells of interest, and then downloading the corresponding cells 
    into a SingleCellExperiment object. 
License: GPL-3
Depends: R (>= 4.2.0)
Imports: dplyr, SummarizedExperiment, SingleCellExperiment, purrr (>=
        1.0.0), BiocGenerics, glue, HDF5Array, DBI, tools, httr, cli,
        assertthat, SeuratObject, Seurat, methods, rlang, stats,
        S4Vectors, tibble, utils, dbplyr (>= 2.3.0), duckdb, stringr
Suggests: zellkonverter, rmarkdown, knitr, testthat, basilisk, arrow,
        reticulate, spelling, forcats, ggplot2,
        tidySingleCellExperiment, rprojroot
Biarch: true
biocViews: AssayDomain, Infrastructure, RNASeq, DifferentialExpression,
        GeneExpression, Normalization, Clustering, QualityControl,
        Sequencing, Transcription, Transcriptomics
Encoding: UTF-8
RoxygenNote: 7.2.3
URL: https://github.com/stemangiola/CuratedAtlasQueryR
BugReports: https://github.com/stemangiola/CuratedAtlasQueryR/issues
VignetteBuilder: knitr
Roxygen: list(markdown = TRUE)
Collate: 'utils.R' 'counts.R' 'dev.R' 'metadata.R' 'seurat.R'
        'unharmonised.R' 'zzz.R'
Language: en-US
PackageStatus: Deprecated
git_url: https://git.bioconductor.org/packages/CuratedAtlasQueryR
git_branch: RELEASE_3_23
git_last_commit: 553d9af
git_last_commit_date: 2026-04-28
Repository: Bioconductor 3.23
Date/Publication: 2026-04-28
NeedsCompilation: no
Packaged: 2026-04-28 21:31:55 UTC; biocbuild
Author: Stefano Mangiola [aut, cre, rev] (ORCID:
    <https://orcid.org/0000-0001-7474-836X>),
  Michael Milton [aut, rev] (ORCID:
    <https://orcid.org/0000-0002-8965-2595>),
  Martin Morgan [ctb, rev],
  Vincent Carey [ctb, rev],
  Julie Iskander [rev],
  Tony Papenfuss [rev],
  Silicon Valley Foundation CZF2019-002443 [fnd],
  NIH NHGRI 5U24HG004059-18 [fnd],
  Victoria Cancer Agency ECRF21036 [fnd],
  NHMRC 1116955 [fnd]
Maintainer: Stefano Mangiola <mangiolastefano@gmail.com>
