# Generated by roxygen2: do not edit by hand

S3method(as.sparse,DelayedMatrix)
export(DATABASE_URL)
export(SAMPLE_DATABASE_URL)
export(get_SingleCellExperiment)
export(get_metadata)
export(get_seurat)
export(get_single_cell_experiment)
export(get_unharmonised_metadata)
importFrom(BiocGenerics,cbind)
importFrom(DBI,dbConnect)
importFrom(DBI,dbDisconnect)
importFrom(HDF5Array,HDF5RealizationSink)
importFrom(HDF5Array,loadHDF5SummarizedExperiment)
importFrom(S4Vectors,DataFrame)
importFrom(Seurat,as.SingleCellExperiment)
importFrom(SeuratObject,as.Seurat)
importFrom(SeuratObject,as.sparse)
importFrom(SingleCellExperiment,SingleCellExperiment)
importFrom(SingleCellExperiment,combineCols)
importFrom(SummarizedExperiment,"assayNames<-")
importFrom(SummarizedExperiment,"colData<-")
importFrom(SummarizedExperiment,colData)
importFrom(assertthat,assert_that)
importFrom(assertthat,has_name)
importFrom(cli,cli_abort)
importFrom(cli,cli_alert_info)
importFrom(cli,cli_alert_success)
importFrom(cli,cli_alert_warning)
importFrom(cli,hash_sha256)
importFrom(dbplyr,remote_con)
importFrom(dbplyr,sql)
importFrom(dplyr,as_tibble)
importFrom(dplyr,collect)
importFrom(dplyr,filter)
importFrom(dplyr,group_by)
importFrom(dplyr,inner_join)
importFrom(dplyr,mutate)
importFrom(dplyr,pull)
importFrom(dplyr,summarise)
importFrom(dplyr,tbl)
importFrom(dplyr,transmute)
importFrom(duckdb,duckdb)
importFrom(glue,glue)
importFrom(httr,GET)
importFrom(httr,HEAD)
importFrom(httr,modify_url)
importFrom(httr,parse_url)
importFrom(httr,progress)
importFrom(httr,stop_for_status)
importFrom(httr,write_disk)
importFrom(methods,as)
importFrom(purrr,imap)
importFrom(purrr,keep)
importFrom(purrr,map)
importFrom(purrr,map_chr)
importFrom(purrr,map_dbl)
importFrom(purrr,map_int)
importFrom(purrr,pmap_chr)
importFrom(purrr,reduce)
importFrom(purrr,set_names)
importFrom(purrr,walk)
importFrom(rlang,.data)
importFrom(stats,setNames)
importFrom(stringr,str_remove_all)
importFrom(stringr,str_replace_all)
importFrom(tibble,column_to_rownames)
importFrom(tools,R_user_dir)
importFrom(utils,head)
importFrom(utils,packageName)
